local lapis = require("lapis")
local date = require("date")
local app = lapis.Application()
local constants = require("constants")
local babycode = require("lib.babycode")
local html_escape = require("lapis.html").escape
local config = require("lapis.config").get()

local db = require("lapis.db")
-- sqlite starts without foreign key enforcement
db.query("PRAGMA foreign_keys = ON")

local util = require("util")

app:enable("etlua")
app.layout = require "views.base"

app.cookie_attributes = function (self, name, value)
  if name == config.session_name then
    if not self.session.queue_delete then
      local expires = date(true):adddays(30):fmt("${http}")
      return "Expires="..expires.."; Path=/; HttpOnly; Secure"
    else
      local expires = date(true):addseconds(-30):fmt("${http}")
      return "Expires="..expires.."; Path=/; HttpOnly; Secure"
    end
  end
end

local function inject_constants(req)
  req.constants = constants
  math.randomseed(os.time())
  req.__cachebust = math.random(99999)
  req.__commit = config.commit
end

local function inject_methods(req)
  req.avatar_url = util.get_user_avatar_url
  req.ntob = function(_, v)
    return util.ntob(v)
  end
  req.PermissionLevelString = constants.PermissionLevelString
  req.infobox_message = function (_, s)
    return util.infobox_message(s)
  end
  
  req.babycode_to_html = function (_, bb)
    return babycode.to_html(bb, html_escape)
  end

  util.pop_infobox(req)
end

app:before_filter(inject_constants)
app:before_filter(inject_methods)

app:include("apps.users", {path = "/user"})
app:include("apps.topics", {path = "/topics"})
app:include("apps.threads", {path = "/threads"})
app:include("apps.mod", {path = "/mod"})
app:include("apps.post", {path = "/post"})
app:include("apps.api", {path = "/api"})

app:get("/", function(self)
  return {redirect_to = self:url_for("all_topics")}
end)

app:get("babycode_guide", "/babycode", function(self)
  self.me = util.get_logged_in_user_or_transient(self)
  self.page_title = "babycode guide"
  return {render = "babycode"}
end)

return app
